import numpy as np
np.seterr(divide='ignore', invalid='ignore')

def is_pattern(pattern, ohlc, ohlc_h1, ohlc_h2):
    open = ohlc[0]
    high = ohlc[1]
    low = ohlc[2]
    close = ohlc[3]

    open_h1 = ohlc_h1[0]
    high_h1 = ohlc_h1[1]
    low_h1 = ohlc_h1[2]
    close_h1 = ohlc_h1[3]

    open_h2 = ohlc_h2[0]
    high_h2 = ohlc_h2[1]
    low_h2 = ohlc_h2[2]
    close_h2 = ohlc_h2[3]

    if pattern == 'doji':
        return (abs(close - open) / (high - low) < 0.05) & \
        (4*(high - np.maximum(close, open)) > (high - low)) & \
        (4*(np.minimum(close, open) - low) > (high - low)) & \
        (high > low)
    else:
        pass

    if pattern == 'gravestone_doji':
        return (abs(close - open) / (high - low) < 0.05) & \
        (5*(high - np.maximum(close, open)) > (high - low)) & \
        (5*(np.minimum(close, open) - low) < (high - low)) & \
        (high > low)
    else:
        pass

    if pattern == 'dragonfly_doji':
        return (abs(close - open) / (high - low) < 0.05) & \
        (5*(high - np.maximum(close, open)) < (high - low)) & \
        (5*(np.minimum(close, open) - low) > (high - low)) & \
        (high > low)
    else:
        pass

    if pattern == 'bullish_spinning_top':
        return (abs(close - open) / (high - low) > 0.1) & \
        (abs(close - open) / (high - low) < 0.2) & \
        (5*(high - np.maximum(close, open)) > (high - low)) & \
        (5*(np.minimum(close, open) - low) > (high - low)) & \
        (close > open)
    else:
        pass

    if pattern == 'bearish_spinning_top':
        return (abs(close - open) / (high - low) > 0.1) & \
        (abs(close - open) / (high - low) < 0.2) & \
        (5*(high - np.maximum(close, open)) > (high - low)) & \
        (5*(np.minimum(close, open) - low) > (high - low)) & \
        (close < open)
    else:
        pass

    if pattern == 'inverted_Hammer':
        return (abs(close - open) / (high - low) > 0.1) & \
        (abs(close - open) / (high - low) < 0.3) & \
        (5*(high - np.maximum(close, open)) > (high - low)) & \
        (10*(np.minimum(close, open) - low) < (high - low)) & \
        (close > open)
    else:
        pass

    if pattern == 'shootingstar':
        return (abs(close - open) / (high - low) > 0.1) & \
        (abs(close - open) / (high - low) < 0.3) & \
        (5*(high - np.maximum(close, open)) > (high - low)) & \
        (10*(np.minimum(close, open) - low) < (high - low)) & \
        (close < open)
    else:
        pass

    if pattern == 'hanging_man':
        return (abs(close - open) / (high - low) > 0.1) & \
        (abs(close - open) / (high - low) < 0.3) & \
        (10*(high - np.maximum(close, open)) < (high - low)) & \
        (5*(np.minimum(close, open) - low) > (high - low)) & \
        (close < open)
    else:
        pass

    if pattern == 'hammer':
        return (abs(close - open) / (high - low) > 0.1) & \
        (abs(close - open) / (high - low) < 0.3) & \
        (10*(high - np.maximum(close, open)) < (high - low)) & \
        (5*(np.minimum(close, open) - low) > (high - low)) & \
        (close > open)
    else:
        pass

    if pattern == 'bullish_marubozu': 
        return (abs(close - open) >= 0.95*(high - low))& \
        (close > open)
    else:
        pass

    if pattern == 'bearish_marubozu':
        return (abs(close - open) >= 0.95*(high - low))& \
        (close < open)
    else:
        pass

    if pattern == 'bullish_belt_hold':
        return (abs(close - open) >= 0.90*(high - low))& \
        (abs(open - low)*100 < (high - low) ) & \
        (high > close) & \
        (close > open)
    else:
        pass

    if pattern == 'bearish_belt_hold':
        return (abs(close - open) >= 0.90*(high - low))& \
        (abs(open - high)*100 < (high - low)) & \
        (low < close) & \
        (close < open)
    else:
        pass

# Double

    if pattern == 'bullish_engulfing':
        return (open_h1 > close_h1) & \
        (close > open) & \
        (open_h1 < close) & \
        (close_h1 > open) & \
        (high_h1 < high) & \
        (low_h1 > low)
    else:
        pass

    if pattern == 'bearish_engulfing':
        return (open_h1 < close_h1) & \
        (close < open) & \
        (open_h1 > close) & \
        (close_h1 < open) & \
        (high_h1 < high) & \
        (low_h1 > low)    
    else:
        pass

    if pattern == 'tweezer_top':
        return (open_h1 < close_h1) & \
        (close < open) & \
        (20*abs(high_h1 - high) < (high - low))
    else:
        pass

    if pattern == 'tweezer_bottom':
        return (open_h1 > close_h1) & \
        (close > open) & \
        (20*abs(low_h1 - low) < (high - low))
    else:
        pass

    if pattern == 'bullish_separating_line':
        return (open_h1 < open) & \
        (open_h1 > close_h1) & \
        (open < close)
    else:
        pass

    if pattern == 'bearish_separating_line':
        return (open_h1 > open) & \
        (open_h1 < close_h1) & \
        (open > close)
    else:
        pass

    if pattern == 'piercing_line':
        return (open_h1 > close_h1) & \
        (open < close) & \
        (close_h1 > open) & \
        (open_h1 > close) & \
        ((open_h1 - close_h1) < 2*(close - close_h1))
    else:
        pass

    if pattern == 'dark_cloud_cover':
        return (open_h1 < close_h1) & \
        (open > close) & \
        (close_h1 < open) & \
        (open_h1 < close) & \
        ((close_h1 - open_h1) < 2*(close_h1 - close))
    else:
        pass

    if pattern == 'mathing_high':
        return (open_h1 < close_h1) & \
        (close > open) & \
        (20*abs(high_h1 - high) < (high - low))
    else:
        pass

    if pattern == 'mathing_low':
        return (open_h1 > close_h1) & \
        (close < open) & \
        (20*abs(low_h1 - low) < (high - low))
    else:
        pass

    if pattern == 'bullish_harami':
        return (open_h1 > close_h1) &\
        (close > open) &\
        (open_h1 > close) &\
        (close_h1  < open) &\
        ((open_h1 - close_h1) > 4*(close - open))
    else:
        pass

    if pattern == 'bearish_harami':
        return (open_h1 < close_h1) &\
        (close < open) &\
        (open_h1 < close) &\
        (close_h1 > open) &\
        ((close_h1 - open_h1) > 4*(open - close))
    else:
        pass

    if pattern == 'bullish_harami_cross':
        return (open_h1 > close_h1) & \
        (open_h1 > close) & \
        (close_h1  < open) & \
        (abs(close - open) / (high - low) < 0.05) & \
        (4*(high - np.maximum(close, open)) > (high - low)) & \
        (4*(np.minimum(close, open) - low) > (high - low)) 
    else:
        pass

    if pattern == 'bearish_harami_cross':
        return (open_h1 < close_h1) & \
        (open_h1 < close) & \
        (close_h1  > open) & \
        (abs(close - open) / (high - low) < 0.05) & \
        (4*(high - np.maximum(close, open)) > (high - low)) & \
        (4*(np.minimum(close, open) - low) > (high - low))
    else:
        pass

    if pattern == 'descending_hawk':
        return (open_h1 < close_h1) & \
        (close > open) & \
        (open_h1 < open) & \
        (close_h1  > close) 
    else:
        pass

    if pattern == 'homing_pigeon':
        return (open_h1 > close_h1) & \
        (close < open) & \
        (open_h1 > open) & \
        (close_h1 < close) 
    else:
        pass


    if pattern == 'bullish_in_neck':
        return (open_h1 < close_h1) & \
        (close < open) & \
        (20*abs(close_h1 - close) < (high - low_h1)) 
    else:
        pass

    if pattern == 'bearish_in_neck':
        return (open_h1 > close_h1) & \
        (close > open) & \
        (20*abs(close_h1 - close) < (high_h1 - low)) 
    else:
        pass

    if pattern == 'bullish_on_neck':
        return (open_h1 < close_h1) & \
        (close < open) & \
        (20*abs(high_h1 - low) < (high - low_h1)) 
    else:
        pass

    if pattern == 'bearish_on_neck':
        return (open_h1 > close_h1) & \
        (close > open) & \
        (20*abs(low_h1 - high) < (high_h1 - low)) 
    else:
        pass

    if pattern == 'bullish_kicking':
        return (open_h1 > close_h1) & \
        (open < close) & \
        ((open_h1 - close_h1) > 0.9*(high_h1 - low_h1)) & \
        ((close_h1 - open_h1) > 0.9*(high - low)) & \
        (open_h1 < open)
    else:
        pass

    if pattern == 'bearish_kicking':
        return (open_h1 < close_h1) & \
        (open > close) & \
        ((close_h1 - open_h1) > 0.9*(high_h1 - low_h1)) & \
        ((open - close) > 0.9*(high - low)) & \
        (open_h1 > open)
    else:
        pass

#triple

    if pattern == 'morning_star':
        return (open_h2 > close_h2) &\
        (3*abs(close_h1 - open_h1) < abs(close_h2 - open_h2)) &\
        (3*abs(close_h1 - open_h1) < abs(close - open)) &\
        (np.maximum(open_h1, close_h1) < close_h2) &\
        (np.maximum(open_h1, close_h1) < open) &\
        (high_h1 > low_h1) &\
        (close > open)
    else:
        pass

    if pattern == 'evening_star':
        return (open_h2 < close_h2) &\
        (3*abs(close_h1 - open_h1) < abs(close_h2 - open_h2)) &\
        (3*abs(close_h1 - open_h1) < abs(close - open)) &\
        (np.minimum(open_h1, close_h1) > close_h2) &\
        (np.minimum(open_h1, close_h1) > open) &\
        (high_h1 > low_h1) &\
        (close < open)
    else:
        pass

    if pattern == 'morning_doji_star':
        return (open_h2 > close_h2) &\
        (abs(close_h1 - open_h1)/(high_h1 - low_h1) < 0.05) &\
        (np.minimum(open_h1, close_h1) < close_h2) &\
        (np.minimum(open_h1, close_h1) < open) &\
        (close > open)
    else:
        pass

    if pattern == 'evening_doji_star':
        return (open_h2 < close_h2) &\
        (abs(close_h1 - open_h1)/(high_h1 - low_h1) < 0.05) &\
        (np.minimum(open_h1, close_h1) > close_h2) &\
        (np.minimum(open_h1, close_h1) > open) &\
        (close < open)
    else:
        pass

    if pattern == 'three_white_soldier':
        return (open_h2 < close_h2) &\
        (2*abs(close_h2 - open_h2) > (high_h2 - low_h2)) &\
        (open_h1 < close_h1) &\
        (2*abs(close_h1 - open_h1) > (high_h1 - low_h1)) &\
        (open < close) &\
        (2*abs(close - open) > (high - low)) &\
        (0.5*(close_h2 + open_h2) < 0.5*(close_h1 + open_h1)) &\
        (0.5*(close_h1 + open_h1) < 0.5*(close + open)) &\
        (close_h1 > close_h2) &\
        (close > close_h1)
    else:
        pass

    if pattern == 'three_black_crow':
        return (open_h2 > close_h2) &\
        (2*abs(close_h2 - open_h2) > (high_h2 - low_h2)) &\
        (open_h1 > close_h1) &\
        (2*abs(close_h1 - open_h1) > (high_h1 - low_h1)) &\
        (open > close) &\
        (2*abs(close - open) > (high - low)) &\
        (0.5*(close_h2 + open_h2) > 0.5*(close_h1 + open_h1)) &\
        (0.5*(close_h1 + open_h1) > 0.5*(close + open)) &\
        (close_h1 < close_h2) &\
        (close < close_h1)
    else:
        pass

    if pattern == 'three_inside_up':
        return (open_h2 > close_h2) &\
        (open_h1 < close_h1) &\
        (open < close) &\
        (open_h1 < open) &\
        ((open + close) > (open_h1 + close_h1)) &\
        (open_h2 > close_h1) &\
        (close_h2 < open_h1)
    else:
        pass

    if pattern == 'three_inside_down':
        return (open_h2 < close_h2) &\
        (open_h1 > close_h1) &\
        (open > close) &\
        (open_h1 > open) &\
        ((open + close) < (open_h1 + close_h1))&\
        (close_h2 > open_h1) &\
        (open_h2 < close_h1)
    else:
        pass

    if pattern == 'deliberation':
        return (open_h2 < close_h2) &\
        (2*abs(close_h2 - open_h2) > (high_h2 - low_h2)) &\
        (open_h1 < close_h1) &\
        (2*abs(close_h1 - open_h1) > (high_h1 - low_h1)) &\
        (open < close) &\
        (3*abs(close - open) < abs(close_h1 - open_h1)) &\
        (0.5*(close_h2 + open_h2) < 0.5*(close_h1 + open_h1)) &\
        (0.5*(close_h1 + open_h1) < 0.5*(close + open)) &\
        (close_h1 > close_h2) &\
        (close > close_h1)
    else:
        pass

    if pattern == 'three_outside_up':
        return (open_h2 > close_h2) &\
        (open_h1 < close_h1) &\
        (open < close) &\
        (open_h1 < open) &\
        ((open + close) > (open_h1 + close_h1)) &\
        (open_h2 < close_h1) &\
        (close_h2 > open_h1)
    else:
        pass

    if pattern == 'three_outside_down':
        return (open_h2 < close_h2) &\
        (open_h1 > close_h1) &\
        (open > close) &\
        (open_h1 > open) &\
        ((open + close) < (open_h1 + close_h1))&\
        (close_h2 < open_h1) &\
        (open_h2 > close_h1)
    else:
        pass

    if pattern == 'bullish_abandoned_baby':
        return (open_h2 > close_h2) &\
        (3*abs(close_h1 - open_h1) < abs(close_h2 - open_h2)) &\
        (3*abs(close_h1 - open_h1) < abs(close - open)) &\
        (np.maximum(open_h1, close_h1) < close_h2) &\
        (np.maximum(open_h1, close_h1) < open) &\
        (high_h1 > low_h1) &\
        (high_h1 < low_h2) &\
        (high_h1 < low) &\
        (close > open)
    else:
        pass

    if pattern == 'bearish_abandoned_baby':
        return (open_h2 < close_h2) &\
        (3*abs(close_h1 - open_h1) < abs(close_h2 - open_h2)) &\
        (3*abs(close_h1 - open_h1) < abs(close - open)) &\
        (np.minimum(open_h1, close_h1) > close_h2) &\
        (np.minimum(open_h1, close_h1) > open) &\
        (high_h1 > low_h1) &\
        (low_h1 > high_h2) &\
        (low_h1 > high) &\
        (close < open)
    else:
        pass

    if pattern == 'bullish_stick_sandwich':
        return (open_h2 > close_h2) &\
        (open_h1 < close_h1) &\
        (open > close) &\
        (open_h1 > close_h2) &\
        (open_h2 < close_h1) &\
        (open > close_h1) &\
        (close < open_h1) &\
        (2*(open_h2 - close_h2) > (high_h2 - low_h2)) &\
        (2*(open - close) > (high - low)) &\
        (20*abs(close_h2 - close) < (high - np.minimum(low_h2,low)))
    else:
        pass

    if pattern == 'bearish_stick_sandwich':
        return (open_h2 < close_h2) &\
        (open_h1 > close_h1) &\
        (open < close) &\
        (open_h1 < close_h2) &\
        (open_h2 > close_h1) &\
        (open < close_h1) &\
        (close > open_h1) &\
        (2*(close_h2 - open_h2) > (high_h2 - low_h2)) &\
        (2*(close - open) > (high - low)) &\
        (20*abs(close_h2 - close) < (low - np.maximum(high_h2, high)))
    else:
        pass
         
    if pattern == 'bullish_side_by_side_white_line':
        return (open_h2 < close_h2) &\
        (open_h1 < close_h1) &\
        (open < close) &\
        (2*(close_h2 - open_h2) > (high_h2 - low_h2)) &\
        (2*(close_h1 - open_h1) > (high_h1 - low_h1)) &\
        (2*(close - open) > (high - low)) &\
        (open_h1 > close_h2) &\
        (20*abs(close_h1 - close) < (np.maximum(high, high_h1)-np.minimum(low, low_h1))) &\
        (20*abs(open_h1 - open) < (np.maximum(high, high_h1)-np.minimum(low, low_h1)))
    else:
        pass

    if pattern == 'bearish_side_by_side_black_line':
        return (open_h2 > close_h2) &\
        (open_h1 > close_h1) &\
        (open > close) &\
        (2*(open_h2 - close_h2) > (high_h2 - low_h2)) &\
        (2*(open_h1 - close_h1) > (high_h1 - low_h1)) &\
        (2*(open - close) > (high - low)) &\
        (open_h1 < close_h2) &\
        (20*abs(close_h1 - close) < (np.maximum(high, high_h1)-np.minimum(low, low_h1))) &\
        (20*abs(open_h1 - open) < (np.maximum(high, high_h1)-np.minimum(low, low_h1)))
    else:
        pass

    if pattern == 'bearish_side_by_side_white_line':
        return (open_h2 > close_h2) &\
        (open_h1 < close_h1) &\
        (open < close) &\
        (2*(open_h2 - close_h2) > (high_h2 - low_h2)) &\
        (2*(close_h1 - open_h1) > (high_h1 - low_h1)) &\
        (2*(close - open) > (high - low)) &\
        (close_h1 < close_h2) &\
        (20*abs(close_h1 - close) < (np.maximum(high, high_h1)-np.minimum(low, low_h1))) &\
        (20*abs(open_h1 - open) < (np.maximum(high, high_h1)-np.minimum(low, low_h1)))
    else:
        pass

    if pattern == 'bullish_side_by_side_black_line':
        return (open_h2 < close_h2) &\
        (open_h1 > close_h1) &\
        (open > close) &\
        (2*(close_h2 - open_h2) > (high_h2 - low_h2)) &\
        (2*(open_h1 - close_h1) > (high_h1 - low_h1)) &\
        (2*(open - close) > (high - low)) &\
        (close_h1 > close_h2) &\
        (20*abs(close_h1 - close) < (np.maximum(high, high_h1)-np.minimum(low, low_h1))) &\
        (20*abs(open_h1 - open) < (np.maximum(high, high_h1)-np.minimum(low, low_h1)))
    else:
        pass
         
    if pattern == 'upside_gap_three':
        return (open_h2 < close_h2) &\
        (open_h1 < close_h1) &\
        (open > close) &\
        (close_h2 < open_h1) &\
        (open > open_h1) &\
        (close < close_h2)
    else:
        pass

    if pattern == 'downside_gap_three':
        return (open_h2 > close_h2) &\
        (open_h1 > close_h1) &\
        (open < close) &\
        (close_h2 > open_h1) &\
        (open < open_h1) &\
        (close > close_h2)
    else:
        pass
         
    if pattern == 'upside_gap_two_crow':
        return (open_h2 < close_h2) &\
        (open_h1 > close_h1) &\
        (open > close) &\
        (open_h1 < open) &\
        (close_h1 > close) &\
        (close_h2 < close) 
    else:
        pass

    if pattern == 'unique_three_river_bottom':
        return (open_h2 > close_h2) &\
        (open_h1 > close_h1) &\
        (open < close) &\
        ((close_h1 - low_h1) > 2*(open_h1 - close_h1)) &\
        (close < close_h1)&\
        (open > low_h1)
    else:
        pass
         
    if pattern == 'bullish_tri_star':
        return (abs(close - open) / (high - low) < 0.05) & \
        (4*(high - np.maximum(close, open)) > (high - low)) & \
        (4*(np.minimum(close, open) - low) > (high - low)) & \
        (high > low)& \
        (abs(close_h1 - open_h1) / (high_h1 - low_h1) < 0.05) & \
        (4*(high_h1 - np.maximum(close_h1, open_h1)) > (high_h1 - low_h1)) & \
        (4*(np.minimum(close_h1, open_h1) - low_h1) > (high_h1 - low_h1)) & \
        (high_h1 > low_h1)& \
        (abs(close_h2 - open_h2) / (high_h2 - low_h2) < 0.05) & \
        (4*(high_h2 - np.maximum(close_h2, open_h2)) > (high_h2 - low_h2)) & \
        (4*(np.minimum(close_h2, open_h2) - low_h2) > (high_h2 - low_h2)) & \
        (high_h2 > low_h2) & \
        (close_h2 > close_h1) & \
        (close > close_h1)
    else:
        pass

    if pattern == 'bearish_tri_star':
        return (abs(close - open) / (high - low) < 0.05) & \
        (4*(high - np.maximum(close, open)) > (high - low)) & \
        (4*(np.minimum(close, open) - low) > (high - low)) & \
        (high > low)& \
        (abs(close_h1 - open_h1) / (high_h1 - low_h1) < 0.05) & \
        (4*(high_h1 - np.maximum(close_h1, open_h1)) > (high_h1 - low_h1)) & \
        (4*(np.minimum(close_h1, open_h1) - low_h1) > (high_h1 - low_h1)) & \
        (high_h1 > low_h1)& \
        (abs(close_h2 - open_h2) / (high_h2 - low_h2) < 0.05) & \
        (4*(high_h2 - np.maximum(close_h2, open_h2)) > (high_h2 - low_h2)) & \
        (4*(np.minimum(close_h2, open_h2) - low_h2) > (high_h2 - low_h2)) & \
        (high_h2 > low_h2) & \
        (close_h2 < close_h1) & \
        (close < close_h1)
    else:
        pass
         
    if pattern == 'three_star_in_the_north':
        return (close_h2 > open_h2) & \
        (close_h1 > open_h1) & \
        (close > open) & \
        (close_h2 - open_h2 > close_h1 - open_h1) & \
        (close_h1 - open_h1 > close - open) & \
        (close_h2 < close_h1) & \
        (close_h1 < close)
    else:
        pass
         
    if pattern == 'three_star_in_the_south':
        return (close_h2 < open_h2) & \
        (close_h1 < open_h1) & \
        (close < open) & \
        (close_h2 - open_h2 < close_h1 - open_h1) & \
        (close_h1 - open_h1 < close - open) & \
        (close_h2 > close_h1) & \
        (close_h1 > close)
    else:
        pass




from .Candlestick import is_pattern
from .report import *
import yfinance as yf
from IPython.core.display import display, HTML
import pandas as pd
from mpl_finance import candlestick2_ochl
import matplotlib.pyplot as plt
import json
import numpy as np
import os


class stockie:
    def __init__(self, stock_name):
        self.stock_name = stock_name
        self.data = {}

        for stock in self.stock_name:
            self.data[stock] = yf.download(stock).reset_index()

        print('{} data loaded successfully'.format(stock_name))
        print(os.getcwd())

    def find_pattern(self, name ='all'):
        all_pattern_data = {}
        single_pattern = ['doji', 'gravestone_doji', 'dragonfly_doji', 'bullish_spinning_top', 'bearish_spinning_top',
                          'inverted_Hammer', 'shootingstar', 'hanging_man', 'hammer', 'bullish_marubozu', 'bearish_marubozu',
                          'bullish_belt_hold', 'bearish_belt_hold']
        double_pattern = ['bullish_engulfing', 'bearish_engulfing', 'tweezer_top', 'tweezer_bottom','bullish_separating_line',
                          'bullish_separating_line', 'piercing_line', 'dark_cloud_cover', 'matching_high', 'matching_low',
                          'bullish_harami', 'bearish_harami', 'bullish_harami_cross', 'bearish_harami_cross', 'descending_hawk',
                          'homing_pigeon', 'bullish_in_neck', 'bearish_in_neck', 'bullish_on_neck','bearish_on_neck','bullish_kicking',
                          'bearish_kicking']
        triple_pattern = ['morning_star', 'evening_star', 'morning_doji_star', 'evening_doji_star', 'three_white_soldier', 'three_black_crow',
                          'three_inside_up', 'three_inside_down', 'deliberation', 'three_outside_up', 'three_outside_down', 'bullish_abandoned_baby',
                          'bearish_abandoned_baby', 'bullish_stick_sandwich', 'bearish_stick_sandwich', 'bullish_side_by_side_white_line', 
                          'bearish_side_by_side_black_line', 'bearish_side_by_side_white_line', 'bullish_side_by_side_black_line',
                          'upside_gap_three', 'downside_gap_three', 'upside_gap_two_crow', 'unique_three_river_bottom', 'bullish_tri_star', 
                          'bearish_tri_star', 'three_star_in_the_north', 'three_star_in_the_south']

        for data_key in self.data:
            data = self.data[data_key]

            self.open = data['Open'].values
            self.high = data['High'].values
            self.low = data['Low'].values
            self.close = data['Close'].values   

            self.open_h1 = data['Open'].shift(1).values
            self.high_h1 = data['High'].shift(1).values
            self.low_h1 = data['Low'].shift(1).values
            self.close_h1 = data['Close'].shift(1).values  

            self.open_h2 = data['Open'].shift(2).values
            self.high_h2 = data['High'].shift(2).values
            self.low_h2 = data['Low'].shift(2).values
            self.close_h2 = data['Close'].shift(2).values  
            

            if name == 'single':
                temp = data.copy()
                for i in single_pattern:
                    temp[i] = is_pattern(i, [self.open, self.high, self.low, self.close],
                                        [self.open_h1, self.high_h1, self.low_h1, self.close_h1],
                                        [self.open_h2, self.high_h2, self.low_h2, self.close_h2])
                    
            if name == 'double':
                temp = data.copy()
                for i in double_pattern:
                    temp[i] = is_pattern(i, [self.open, self.high, self.low, self.close],
                                        [self.open_h1, self.high_h1, self.low_h1, self.close_h1],
                                        [self.open_h2, self.high_h2, self.low_h2, self.close_h2])
                    

            if name == 'triple':
                temp = data.copy()
                for i in triple_pattern:
                    temp[i] = is_pattern(i, [self.open, self.high, self.low, self.close],
                                        [self.open_h1, self.high_h1, self.low_h1, self.close_h1],
                                        [self.open_h2, self.high_h2, self.low_h2, self.close_h2])

            if name == 'all':
                temp = data.copy()
                for i in single_pattern + double_pattern + triple_pattern:
                    temp[i] = is_pattern(i, [self.open, self.high, self.low, self.close],
                                        [self.open_h1, self.high_h1, self.low_h1, self.close_h1],
                                        [self.open_h2, self.high_h2, self.low_h2, self.close_h2])

            all_pattern_data[data_key] = temp
        return all_pattern_data

    def get_candlestick_report(self,shape=20):
        clear_output()
        return display(HTML(generate_HTML_CS_overview_new(self.find_pattern(), shape= 20)))



# compilation 
import yfinance as yf
from IPython.core.display import display, HTML
import pandas as pd
from mpl_finance import candlestick2_ochl
import matplotlib.pyplot as plt
import json
import numpy as np
import os

import matplotlib.pyplot as plt
import base64
from io import BytesIO

local_path = '/'.join(yf.__file__.split('/')[:-2]) + '/Stockie/'

def create_label(df, timegap):
    return np.array(((df['Close'].shift(-timegap) >= df['Close'])*1).tolist()[:-timegap] + timegap*[np.nan])
    
def create_rate(df, timegap):
    return np.array((df['Close'].shift(-timegap) - df['Close'])/df['Close'])

def transform(ma,mi,x):
    return -321*x/(ma-mi) + (45*mi - 366*ma)/(mi-ma)

def plot_ma_mi_scaler(data_temp, shape = 20):
    range = data_temp[['Open','High','Low','Close']].max().max() - data_temp[['Open','High','Low','Close']].min().min()
    if range >2000:
        scaler = 500
    elif range > 500:
        scaler = 50
    else:
        scaler = 10
    return scaler

def generate_image(data, shape = 20, name = ""):
    data_temp = data.copy().tail(shape).reset_index()

    my_dpi = 144
    fig = plt.figure(figsize=(800/my_dpi, 400/my_dpi), dpi=my_dpi)
    ax = fig.add_subplot(1,1,1)
    fig.subplots_adjust(bottom=0.1,left=0.1,right=.9,top=.9)
    candlestick2_ochl(ax, data_temp['Open'], data_temp['Close'], data_temp['High'], data_temp['Low'], 
                    width = 0.5, colordown='r', colorup = 'g')
    plt.ylim([data_temp[['Open','High','Low','Close']].min().min() - plot_ma_mi_scaler(data_temp), 
              data_temp[['Open','High','Low','Close']].max().max() + plot_ma_mi_scaler(data_temp)])
    ax.yaxis.tick_right()
    
    # awas ini kalau ganti shape
    plt.xticks([0,5,10,15,19], data_temp.loc[[0,5,10,15,19],'Date'].astype('str').values.tolist(), fontsize=7)
    tmpfile = BytesIO()
    fig.savefig(tmpfile, format='png')
    plt.close()
    encoded = base64.b64encode(tmpfile.getvalue()).decode('utf-8')

    html ='<img src=\'data:image/png;base64,{encoded}\'>'.format(encoded = encoded, name = name)

    return html[:-1] + ' width="800" height="400" >'

def create_pattern_info_dict(data, label_length = 10, name = " "):
    data_temp = data.copy()

    pattern = data_temp.columns.drop(['Date','Open', 'High', 'Low', 'Close', 'Adj Close', 'Volume'])

    for i in [1,2,3,4,5,6,7,8,9,10]:
        data_temp['label_{}'.format(i)] = create_label(data_temp, i) 
        data_temp['rate_{}'.format(i)] = create_rate(data_temp, i)

    pattern_info_dict = {}
    for i in pattern:
        try:
            pattern_info_dict[i+ " " + name] = [data_temp.groupby(i).count()['Date'][True]
                                    ,np.round(data_temp.groupby(i).mean()[['label_{}'.format(j) for j in range(1,label_length + 1)]].T[True].values*100,2).tolist()]
        except:
            continue

    return pattern_info_dict

def html_interactive_candlestick(data, top, left, shape = 20, name = ""):
    temp = data.copy().tail(shape).reset_index()
    mi ,ma = temp[['Open','High','Low','Close']].min().min() - plot_ma_mi_scaler(temp), temp[['Open','High','Low','Close']].max().max() + plot_ma_mi_scaler(temp)
    html_style_str = ''
    html_body_str = ''
    for i in range(shape):
        html_style_str += '''
.area-{name}-{label}{{
    height: {height};
    width: {width};
    top: {top};
    left: {left};
    border: {border};
    position: absolute;
}}
        '''.format(label = str(i+1),
           height = str(transform(ma,mi,temp.loc[i,'Low']) - transform(ma,mi,temp.loc[i,'High'])) + 'px',
           width = '13px',
           top = str(transform(ma,mi,temp.loc[i,'High']) + top) + 'px',
           left = str(107 + i*29.15 + left) + 'px',
           border = '0px solid #555',
           name = name)
        
        html_body_str += '''
<div class="invisible-area area-{name}-{label}">
    <div class="tooltip"><pre>
Date :{Date}
Open :{Open}
High :{High}
Low  :{Low}
Close:{Close}
Vol  :{Vol}</pre>
    </div>
</div>
        '''.format(label = str(i+1),
                   Date = str(temp.loc[i,'Date'])[:10],
                   Open = temp.loc[i,'Open'],
                   High = temp.loc[i,'High'],
                   Low = temp.loc[i,'Low'],
                   Close = temp.loc[i,'Close'],
                   Vol = temp.loc[i,'Volume'],
                   name = name)

    return html_style_str, html_body_str

def create_pattern_det_dict(data, shape = 20, name = ""):
    data_temp = data.copy().tail(shape).reset_index()

    horizontal_sum = data_temp[data_temp.columns.drop(['index', 'Date', 'Open', 'High', 'Low', 'Close', 'Adj Close', 'Volume'])].sum(axis = 1)

    pattern_det_dict = {}
    for i in range(shape):
        if horizontal_sum[i] != 0:
            pattern_det_dict[i] = (data_temp == True).iloc[i][(data_temp == True).iloc[i]==True].index.tolist()
        else:
            continue
    
    return pattern_det_dict

def pattern_detection(data,pat_dict, top, left, shape = 20, name = ""):
    
    with open(local_path + "pattern_img.json", 'r') as f:
        img_dict = json.load(f)
    with open(local_path + "pattern_type.json", 'r') as f:
        type_dict = json.load(f)    

    pattern_str_html_style = ''
    pattern_str_html_body = ''
    temp = data.copy().tail(shape).reset_index()
    mi ,ma = temp[['Open','High','Low','Close']].min().min() - plot_ma_mi_scaler(temp), temp[['Open','High','Low','Close']].max().max() + plot_ma_mi_scaler(temp)
    for i in pat_dict:
        pattern_str_html_style += """
.area-{name}-pattern-{label}{{
    height: {height};
    width: {width};
    top: {top};
    left: {left};
    border: {border};
    position: absolute;
}}
        """ .format(label = str(i+1),
           height = '30' + 'px',
           width = '13px',
           top = str(transform(ma,mi,temp.loc[i,'High']) - 30 + top) + 'px',
           left = str(107 + i*29.15 + left) + 'px',
           border = '0px solid #555',
           name = name)
        
        image_text = ''
        for pat in pat_dict[i]:
            image_text +="""
        <pr>
        <p style="text-align:center;width:150px;">
        <b>{pattern}</b><br>
        {image}<br>
        {types}<br>
        <p class="{pattern_id}" style="text-align:center">{text}</p>
        </p>
        </pr>
            """.format(image = img_dict[pat][:-1] + ' style= "border:1px solid #555">',
                       types = type_dict[pat].replace('BU', 'Bullish').replace('BE', 'Bearish').replace('CO', 'Continuation').replace('RE', 'Reversal').replace('I', 'Inconclusive'),
                   pattern = pat.replace("_", " "),
                   pattern_id = pat + " " + name,
                   text = ''
                   )

        pattern_str_html_body += """
<div class = "invisible-area area-{name}-pattern-{label}" ><p class = "pattern"> ! </p>
    <div class="tooltip">
        {image_text}
    </div>
</div>
        """.format(label = str(i+1),
                   image_text = image_text,
                   name = name)
    return pattern_str_html_style, pattern_str_html_body

def create_stacked(real_html, body, pattern_body, style, pattern_style):
    full_style = ""
    full_overview_body = ""

    for i in real_html:
        full_style += style[i] + pattern_style[i] 
        full_overview_body += """
<div id = "{name}">
    {a}  
    {b}  
    {c} 
</div>
""".format(name = i,
           a = real_html[i],
           b = body[i],
           c = pattern_body[i])

    return full_style, full_overview_body

def create_option_image(data):
    option_image = '<select id="mySelect" onchange="changeimage(this.value)" style="position:absolute;">'
    for name in data:
        option_image += '\n <option value="{name}">{name}</option>'.format(name = name)
    option_image += '\n </select>'
    return option_image

def generate_HTML_CS_overview_new(data_full, shape = 20, top = 104, left = 13):
    real_html = {}
    pattern_info_dict = {}
    pattern_det_dict = {}
    style = {}
    body = {}
    pattern_style = {} 
    pattern_body = {}
    for stock_name  in data_full:
        stock_name = stock_name.replace('.JK', '')
        df = data_full[stock_name + '.JK'].copy()
        real_html[stock_name] = generate_image(df, shape = 20, name = stock_name)
        pattern_info_dict.update(create_pattern_info_dict(df, name = stock_name))
        pattern_det_dict[stock_name] = create_pattern_det_dict(df)
        style[stock_name], body[stock_name] = html_interactive_candlestick(data = df ,top = top, left = left, shape = 20, name = stock_name)
        pattern_style[stock_name], pattern_body[stock_name] = pattern_detection(data = df,pat_dict = pattern_det_dict[stock_name], 
                                                                                top = top, left = left, name = stock_name)
        
    stock_name_list = [i[:-3] for i in list(data_full.keys())]
    option_image = create_option_image(data = stock_name_list)
    full_style, full_overview_body = create_stacked(real_html, body, pattern_body, style, pattern_style)

    return '''
<!DOCTYPE html>
<html>
    <head>
        <meta name="viewport" content="width=device-width, initial-scale=1">
        <style>
        body {font-family: Arial;}
        /* Style the tab */
        .tab {
            overflow: hidden;
            border: 1px solid #ccc;
            background-color: #f1f1f1;
            }
        /* Style the buttons inside the tab */
        .tab button {
            background-color: inherit;
            float: left;
            border: none;
            outline: none;
            cursor: pointer;
            padding: 14px 16px;
            transition: 0.3s;
            font-size: 17px;
            }

        /* Change background color of buttons on hover */
        .tab button:hover {
            background-color: #ddd;
            }

        /* Create an active/current tablink class */
        .tab button.active {
            background-color: #ccc;
            }

        /* Style the tab content */
        .tabcontent {
            display: none;
            padding: 6px 12px;
            border: 1px solid #ccc;
            border-top: none;
            }
        table {
            width: border-collapse: collapse;
            border : 1px solid #ddd
            }
        tr:nth-child(even) {
            background-color: #dddddd;
            }
        th {
            padding: 10px;
            text-align: left;
            background-color: #4CAF50
            }
        td {
            padding: 10px;
            text-align: left;
            }
        
        .invisible-area {
            position: absolute;
            }

        .tooltip {
        display: none;
        background: white;
        border-radius: 5px;
        border: 1px black solid;
        position: absolute;
        background-color: white;
        border-radius: 5px;
        padding: 1px;
        z-index: 1;
        top: 0px;
        left: 150%;
        overflow:auto;
        max-height:200px;
        }

        .invisible-area .tooltip::after {
        content: "";
        position: absolute;
        top: 50%;
        right: 100%;
        margin-top: -5px;
        border-width: 5px;
        border-style: solid;
        border-color: transparent black transparent transparent;
        
        }

        .invisible-area:hover .tooltip {
        display: block;
        }

        .pattern {
            background-color:powderblue;
            border-radius: 5px;
            text-align:center;
        }

        ''' + full_style +'''

        </style>
        
</head>
<body>

<h2>Report</h2>

<div class="tab">
    <button class="tablinks" onclick="openreport(event, 'Overview')">Overview</button>
    <button class="tablinks" onclick="openreport(event, 'Top Gain')">Top Gain</button>
    <button class="tablinks" onclick="openreport(event, 'Top Loss')">Top Loss</button>
</div>

<div id="Overview" class="tabcontent">

''' + option_image +'''

''' + full_overview_body +'''

<input type="range" name="rangeInput" style ="position:absolute;left:93px; top:124px;" min="1" max="10" value = "5" onchange="updateTextInput(this.value);">
<input type="text" id="textInput" value="5" size="1" style ="position:absolute;left:233px; top:124px;" >



</div>
<div id="Top Gain" class="tabcontent">
    <h3>Top Gain</h3>
    <table>
        <tr>
            <th>Pattern</th>
            <th>Accuracy</th>
            <th>Rate</th>
        </tr>
        </tr>
    </table>
</div>

<div id="Top Loss" class="tabcontent">
    <h3>Top Loss</h3>
    <table>
        <tr>
            <th>Pattern</th>
            <th>Accuracy</th>
            <th>Rate</th>
        </tr>
    </table>
</div>

<p id = "demo"><p>

<script>

var acc_dicts = ''' + str(pattern_info_dict) + ''';
var type_dicts = ''' + str(type_dict) + ''';
var stock_name = ''' + str(stock_name_list) + ''';

changeimage(stock_name[0])
updateTextInput(5)

function updateTextInput(val) {
    document.getElementById('textInput').value=val;
    for (var key in acc_dicts) {
        var x = document.getElementsByClassName(key);
        var i;
        for (i = 0; i < x.length; i++) {
            if ((type_dicts[key.slice(0,-5)] == "BU RE")||(type_dicts[key.slice(0,-5)] == "BU CO")||(type_dicts[key.slice(0,-5)] == "I")){
                x[i].innerHTML = "<p style='color:green;font-size:20px;'>&#10138;<span style='color:black;font-size:15px;'>" 
                + acc_dicts[key][1][val-1] + "%</span></p>" +  (acc_dicts[key][0]*acc_dicts[key][1][val-1]/100).toFixed(0) + 
                " <span style='color:green;font-weight:10px'>Up</span> out of " + acc_dicts[key][0];
                }
            else{
                x[i].innerHTML = "<p style='color:red;font-size:20px;'>&#10136;<span style='color:black;font-size:15px;'>" 
                + (100 - acc_dicts[key][1][val-1]).toFixed(2) + "%</span></p>" +(acc_dicts[key][0]*(100-acc_dicts[key][1][val-1])/100).toFixed(0) + 
                " <span style='color:red;font-weight:10px'>Down</span> out of " + acc_dicts[key][0];
            }
            }
    }
        }

function openreport(evt, cityName) {
  var i, tabcontent, tablinks;
  tabcontent = document.getElementsByClassName("tabcontent");
  for (i = 0; i < tabcontent.length; i++) {
    tabcontent[i].style.display = "none";
  }
  tablinks = document.getElementsByClassName("tablinks");
  for (i = 0; i < tablinks.length; i++) {
    tablinks[i].className = tablinks[i].className.replace(" active", "");
  }
  document.getElementById(cityName).style.display = "block";
  evt.currentTarget.className += " active";
}

function changeimage(val){
    for (i = 0; i < stock_name.length; i++){
        document.getElementById(stock_name[i]).style.display = "none";
    }
    document.getElementById(val).style.display = "block";
}
</script>
   
</body>
</html> 

'''



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="stockie", 
    version="0.0.12",
    author="Suparjo Tamin",
    author_email="suparjo.tamin@gmail.com",
    description="A package to identify stock candlestick pattern",
    long_description=long_description,
    url="https://github.com/suparjotamin/stocky",
    packages=setuptools.find_packages(),
    package_data = {
        '': ['pattern_img.json','pattern_type.json']
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=['yfinance', 'mpl_finance'],
)
